#!/bin/env python
"""
./plots.py [--help] file
Plots the supplied data file.
"""

import tables
import netCDF4
import numpy
import matplotlib
import pylab
import sys
import getopt

xName =    "x Coordinates"
dataName = "Data"

def readHDF5file(filename):
    data = None
    xData = None

    h5file=tables.openFile(filename,mode="r")
    for node in h5file.walkNodes('/',classname='Array'):
      if (node.name == dataName):
        data = node.read()
      if (node.name == xName):
        xData = node.read()

    return ("HDF5", data,xData)

def readNetCDF4file(filename):
    data = None
    xData = None
    file=netCDF4.Dataset(filename,"r")

    if dataName in file.variables:
        data = file.variables[dataName][:]
    if xName in file.variables:
        xData = file.variables[xName][:]

    file.close()
    return ("NetCDF4", data,xData)

def readBinaryFile(filename):
    data = numpy.memmap(filename, dtype='float64', mode='r')
    return ("Binary", data, None)

def getData(filename="data.h5"):
    data = None
    xData  = None
    ext = ((filename.split("."))[-1]).lower()
    if ext == "h5" or ext == "hdf5" or ext == "hdf" :
        (filetype, data, xData) = readHDF5file(filename)
    elif ext == "nc" or ext == "nc4" or ext == "netcdf"  or ext=="ncdf":
        (filetype, data, xData) = readNetCDF4file(filename)
    else:
        (filetype, data, xData) = readBinaryFile(filename)
    return (filetype,data, xData)


def plotArray(filename="data.h5"):
    (filetype, data, xData) = getData(filename)

    if data is None and xData is None:
        print "No Data in file "+filename
        return

    print "Plotting ",filename

    nx = None
    if not data is None:
        nx = len(data)

    if xData is None:
        hdims = numpy.arange(0,nx)
    else:
        hdims = xData

    fig = pylab.figure(1)
    ax = fig.add_subplot(111)
    pylab.plot(hdims,data)
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_title(filetype+' ' +filename)

    pylab.show()


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
             raise Usage(__doc__)

        for o,a in opts:
            if o in ("-h", "--help"):
                print __doc__
                sys.exit(0)
        
        if args == []:
            plotArray()
        for arg in args:
            plotArray(arg)            
        sys.exit(0)

    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2


if __name__ == "__main__":
    sys.exit(main())

