#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <mpi.h>
#include "adios.h"

#include "rundata.h"
#include "options.h"

void writeadiosfile(rundata_t rundata, float *x, double *data) {
	int         adios_err=0;
	uint64_t    adios_groupsize, adios_totalsize;
	int64_t     adios_handle;
    MPI_Comm    comm = MPI_COMM_WORLD;

	adios_init ("1d.xml");
	adios_open (&adios_handle, "ArrayData", rundata.filename, "w", &comm);
	#include "gwrite_ArrayData.ch"

    if (adios_err)
        fprintf(stderr,"Error doing adios write.\n");

	adios_close (adios_handle);
}


int main(int argc, char **argv) {
    float *x;
    double *data;
    rundata_t rundata;
    int rank, size;

    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD,&size);
    MPI_Comm_rank(MPI_COMM_WORLD,&rank);

    /*  
     * set default values for parameters, then check what the user says
     * with the options 
     */
    
    rundata.globaln = 100;
    rundata.nprocs = size;
    rundata.rank = rank;
    strcpy(rundata.filename,"sine.nc");

    /* get options */
    get_options(argc,argv,&rundata);

    printf("[%d]: Assigned to region (%d,%d) \n", rank, rundata.start, rundata.start+rundata.localn-1);
    
    /* set up coordinates */
    x = (float *)malloc(rundata.localn * sizeof(float));
    for (int i=0; i<rundata.localn; i++) 
        x[i] = (i+rundata.start)*rundata.dx;
    
    /* set up variable */
    data = (double *)malloc(rundata.localn * sizeof(double));
    for (int i=0; i<rundata.localn; i++) 
        data[i] = sin(x[i]);
    
    writeadiosfile(rundata, x, data);
    printf("[%d]: Wrote file\n", rank);

    free(data);
    free(x);

    MPI_Finalize();
    return 0;
}  
