#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <mpi.h>

#include "rundata.h"
#include "options.h"

void writempiiofile(rundata_t rundata, double *data) {
    MPI_Offset offset;
    MPI_File   file;
    MPI_Status status;
    MPI_Datatype viewtype;

    MPI_File_open(MPI_COMM_WORLD, rundata.filename,
                  MPI_MODE_CREATE|MPI_MODE_WRONLY,
                  MPI_INFO_NULL, &file);

    offset = 0;

    int globalsizes[1] = {rundata.globaln};
    int localsizes [1] = {rundata.localn};
    int starts     [1] = {rundata.start};

    MPI_Type_create_subarray( 1, globalsizes, localsizes, starts, MPI_ORDER_C, MPI_DOUBLE, &viewtype );
    MPI_Type_commit(&viewtype);

    MPI_File_set_view(file, offset,  MPI_DOUBLE, viewtype,
                           "native", MPI_INFO_NULL);

    MPI_File_write_at_all(file, offset*sizeof(double), data, rundata.localn, 
                          MPI_DOUBLE, &status);

    MPI_File_close(&file);
} 

int main(int argc, char **argv) {
    int ierr, rank, size;
    double *data;
    rundata_t rundata;

    rundata.globaln = 100;
    strcpy(rundata.filename,"sine-mpiio.dat");

    ierr = MPI_Init(&argc, &argv);
    ierr|= MPI_Comm_size(MPI_COMM_WORLD, &size);
    ierr|= MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    
    rundata.nprocs = size;
    rundata.rank = rank;

    get_options(argc,argv,&rundata);

    printf("[%d]: Assigned to region (%d,%d) \n", rank, rundata.start, rundata.start+rundata.localn-1);

    data = malloc(rundata.localn * sizeof(double));

    for (int i=0; i<rundata.localn; i++)
        data[i] = sin((rundata.start+i)*rundata.dx);

    writempiiofile(rundata, data);

    free(data);
    MPI_Finalize();
    return 0;
}
